package service

import (
	"errors"
	"fonchain-artshow/cmd/dao"
	"fonchain-artshow/cmd/model"
	"fonchain-artshow/pb/artShow"
	"fonchain-artshow/pkg/db"
	"fonchain-artshow/pkg/m"
	"fonchain-artshow/pkg/serializer"
)

// 创建 画展包 及 关联画作
func CreateArtShowWithArtworkPrice(in *artShow.SaveShowReq) (err error, showUID string) {
	artShowM := serializer.BuildArtShowM(in)

	tx := db.DbArtShow.Begin()

	err = dao.SaveArtShow(tx, artShowM)
	if err != nil {
		tx.Rollback()
		return
	}

	if len(in.ShowArtwork) > 0 {
		artworks := serializer.BuildShowArtworkM(in.ShowArtwork, artShowM.ShowUID)

		artworks = serializer.CalcPrice(artShowM.Price, artShowM.Ruler, artworks)

		err, artworks = serializer.CalcReward(artworks, artShowM.Reward)
		if err != nil {
			return
		}
		for i := 0; i < len(artworks); i++ {
			err = dao.SaveArtworkPrice(tx, artworks[i])
			if err != nil {
				tx.Rollback()
				return
			}
		}
	}

	err = tx.Commit().Error

	return err, artShowM.ShowUID
}

// 更新 画展包 及 关联画作
func UpdateArtShowWithArtworkPrice(in *artShow.SaveShowReq) (err error, showUID string) {
	tx := db.DbArtShow.Begin()
	artShowM := serializer.BuildArtShowM(in) // 构建画展包数据
	artworks := make([]*model.ArtworkPrice, 0)

	// 查询是否已有画作存在
	err, artworkPrices := dao.ArtworkPriceList(in.ShowUID)
	if err != nil {
		return
	}
	if len(artworkPrices) > 0 {
		artworks = append(artworks, artworkPrices...)
	}

	// 判断是否有新增画作
	if len(in.ShowArtwork) > 0 {
		showArtwork := serializer.BuildShowArtworkM(in.ShowArtwork, in.ShowUID)
		artworks = append(artworks, showArtwork...)
	}

	// 更新 画作
	if len(artworks) > 0 {
		if in.Price != 0 {
			artworks = serializer.CalcPrice(artShowM.Price, artShowM.Ruler, artworks)
		}
		if in.Reward != 0 {
			err, artworks = serializer.CalcReward(artworks, artShowM.Reward)
			if err != nil {
				return
			}
		}

		for i := 0; i < len(artworks); i++ {
			if artworks[i].ArtworkPriceUID != "" {
				// 更新画作价格
				err := dao.UpdateArtworkPrice(tx, artworks[i])
				if err != nil {
					tx.Rollback()
					return err, ""
				}
			} else {
				err = dao.SaveArtworkPrice(tx, artworks[i])
				if err != nil {
					tx.Rollback()
					return err, ""
				}
			}
		}
	}

	// 删除旧画作
	if len(in.DelShowArtwork) > 0 {
		del := make([]string, 0)
		for i := 0; i < len(in.DelShowArtwork); i++ {
			del = append(del, in.DelShowArtwork[i].ArtworkPriceUID)
		}
		err = dao.DelArtworkPrice(tx, del)
		if err != nil {
			tx.Rollback()
			return err, ""
		}
	}

	// 更新画展包
	err = dao.UpdateArtShow(tx, artShowM)
	if err != nil {
		tx.Rollback()
		return
	}

	err = tx.Commit().Error
	return err, artShowM.ShowUID
}

// 删除画展包
func DelArtShow(in *artShow.DelShowReq) (err error) {
	tx := db.DbArtShow.Begin()
	for i := 0; i < len(in.ShowUID); i++ {
		queryShowRelErr, show_rel := dao.QueryShowRel_showUID(in.ShowUID[i])
		if queryShowRelErr == nil && show_rel.ID == 0 {
			delArtShowErr := dao.DelArtShow(tx, in.ShowUID[i])
			if err != nil {
				tx.Rollback()
				return delArtShowErr
			}
			delArtworkPriceErr := dao.DelArtworkPrice_showUID(tx, in.ShowUID[i])
			if err != nil {
				tx.Rollback()
				return delArtworkPriceErr
			}
		} else if show_rel.ShowUID != "" {
			tx.Rollback()
			return errors.New("存在画展包被使用,无法删除")
		}

	}
	err = tx.Commit().Error
	return
}

// 画展包列表
func ArtShowList(in *artShow.ShowListReq) (err error, out *artShow.ShowListRes) {
	out = new(artShow.ShowListRes)
	artShows := make([]*model.ArtShow, 0)
	err, out.Total, artShows = dao.ArtShowList(in)
	if err != nil {
		return
	}
	err, out.TotalArtistNum, out.TotalPackageNum, _, _, _ = dao.ShowStatistical(m.ARTSHOW_PASS)
	if err != nil {
		return
	}
	out.Data = serializer.BuildArtShowListRes(artShows)
	return
}

// 画展包关联画作信息
func ShowArtworkInfo(in *artShow.ShowDetailReq) (err error, out *artShow.ShowArtworkDetailRes) {
	out = new(artShow.ShowArtworkDetailRes)
	artworkPriceS := make([]*model.ArtworkPrice, 0)
	err, artworkPriceS = dao.ArtworkPriceList(in.ShowUID)
	if err != nil {
		return
	}
	out.Data = serializer.BuildShowArtworkRpc(artworkPriceS)
	return
}

// 画展包详情
func ShowDetail(in *artShow.ShowDetailReq) (err error, out *artShow.ShowDetailRes) {
	out = new(artShow.ShowDetailRes)
	show := new(model.ArtShow)
	err, show = dao.QueryArtShow(in.ShowUID)
	if err != nil {
		return
	}
	out.Data = serializer.BuildArtShowRpc(show)
	return
}

func ArtShowListWithApply(in *artShow.ShowListReq) (err error, out *artShow.ShowListRes) {
	out = new(artShow.ShowListRes)
	artShows := make([]*model.ArtShow, 0)
	err, out.Total, artShows = dao.ArtShowListByApplyStatus(in)
	err, out.TotalArtistNum, out.TotalPackageNum, _, _, _ = dao.ShowStatistical(m.ARTSHOW_PASS)
	if err != nil {
		return
	}
	out.Data = serializer.BuildArtShowListRes(artShows)
	return
}

func ShowStatisticalInfo(in *artShow.ShowStatisticalInfoReq) (err error, out *artShow.ShowStatisticalInfoRes_Num) {
	out = new(artShow.ShowStatisticalInfoRes_Num)
	err, out.ArtistNum, out.PackageNum, out.TotalNum, out.NotShowNum, out.ShowHisNum = dao.ShowStatistical(in.IsShow)
	if err != nil {
		return
	}
	return
}
